import datetime

class Dates:

    def __init__(self, arrival_date, length_of_stay):
        self._arrival_date = self.__validate_arrival_date(arrival_date)
        self.length_of_stay = self.__validate_length_of_stay(length_of_stay) 
        self._request_dates = None
        self._stay_dates = None

    def __validate_arrival_date(self, date_input):
        try:
            arrival_date = datetime.datetime.strptime(date_input, "%m-%d-%Y")
        except ValueError:
            raise ValueError(f'"{date_input}" is not a valid date of the form mm-dd-yyyy')
        if datetime.datetime.now() > arrival_date:
            raise ValueError(f'"{date_input}" is not a future date.')
        return arrival_date

    def __validate_length_of_stay(self, length_input):
        if length_input.isnumeric():
            return int(length_input)
        else:
            raise ValueError('"{length_input}" is not an integer.')

    @property
    def request_dates(self):
        if not self._request_dates == None:
            return self._request_dates
        else:
            request_dates = []
            first_day = self._arrival_date.replace(day=1)
            last_day = self._arrival_date + datetime.timedelta(days=self.length_of_stay)
            for month in range(first_day.month, (last_day.month + 1)):
                request_date = first_day.replace(month=month)
                request_date = request_date.strftime("%Y-%m-%dT00:00:00.000Z")
                request_dates.append(request_date)
            self._request_dates = request_dates
            return self._request_dates
    
    @property
    def stay_dates(self):
        if not self._stay_dates == None:
            return self._stay_dates
        else:
            stay_dates = []
            stay_range = range(self.length_of_stay)
            for i in stay_range:
                date = self._arrival_date + datetime.timedelta(days=i)
                date = date.strftime("%Y-%m-%dT00:00:00Z")
                stay_dates.append(date)
            self._stay_dates = stay_dates
            return self._stay_dates



import os 

from pathlib import Path


def get_project_root() -> Path:
    return Path(__file__).parent.parent

PROJECT_ROOT = get_project_root()    

DOWNLOAD_FOLDER = os.path.join(PROJECT_ROOT, "assets")
DOWNLOAD_PATH = os.path.join(DOWNLOAD_FOLDER, "RIDBExport.zip")
FACILITIES_CSV_PATH = os.path.join(DOWNLOAD_FOLDER, "Facilities_API_v1.csv")

DATA_FOLDER = os.path.join(PROJECT_ROOT, "data")
FACILITIES_DB = os.path.join(DATA_FOLDER, "rgov.db")

CONFIG_DIR = os.getenv("XDG_CONFIG_HOME", os.path.join(Path.home(), ".config"))
AUTH_FILE = os.path.join(CONFIG_DIR, "rgov", "auth.txt") 
LOG_FILE = os.path.join(Path.home(), ".rgov.log")



import sqlite3

from contextlib import closing
from typing import Generator

from rgov import locations

def search(search_terms: list, 
           descriptions=False) -> Generator[tuple[str, str], None, None]:
    if not isinstance(search_terms, list):
         raise ValueError("arg: search_terms not a list")

    query = [f"%{term.lower()}%" for term in search_terms]

    num_search_terms = len(query)

    if descriptions == True:
        column = 'description'
    else:
        column = 'name'

    separator = f" AND {column} LIKE "

    sql_statement = f"""SELECT *
                     FROM campgrounds 
                     WHERE ({column} LIKE {separator.join(['?' for _ in query])}) 
                     ORDER BY name;"""

    with closing(sqlite3.connect(locations.FACILITIES_DB)) as con, con, \
            closing(con.cursor()) as cur:
        for row in cur.execute(sql_statement, query):
            yield row[1].title(), row[0]



import contextlib
import csv
import os
import urllib.request
import re
import shutil
import sqlite3
import zipfile

from collections import Counter

from cleo import Command
from cleo.helpers import option

from rgov import locations, pushsafer

STOPWORDS = Counter(["a", "about", "above", "after", "again", "against", "ain", "all", "am", "an", "and", "any", "are", "aren", "aren't", "as", "at", "be", "because", "been", "before", "being", "below", "between", "both", "but", "by", "campground", "campsite", "can", "couldn", "couldn't", "d", "did", "didn", "didn't", "do", "does", "doesn", "doesn't", "doing", "don", "don't", "down", "during", "each", "few", "for", "from", "further", "had", "hadn", "hadn't", "has", "hasn", "hasn't", "have", "haven", "haven't", "having", "he", "her", "here", "hers", "herself", "him", "himself", "his", "how", "i", "if", "in", "into", "is", "isn", "isn't", "it", "it's", "its", "itself", "just", "ll", "m", "ma", "me", "mightn", "mightn't", "more", "most", "mustn", "mustn't", "my", "myself", "needn", "needn't", "no", "nor", "not", "now", "o", "of", "off", "on", "once", "only", "or", "other", "our", "ours", "ourselves", "out", "over", "own", "re", "s", "same", "shan", "shan't", "she", "she's", "should", "should've", "shouldn", "shouldn't", "so", "some", "such", "t", "than", "that", "that'll", "the", "their", "theirs", "them", "themselves", "then", "there", "these", "they", "this", "those", "through", "to", "too", "under", "until", "up", "ve", "very", "was", "wasn", "wasn't", "we", "were", "weren", "weren't", "what", "when", "where", "which", "while", "who", "whom", "why", "will", "with", "won", "won't", "wouldn", "wouldn't", "y", "you", "you'd", "you'll", "you're", "you've", "your", "yours", "yourself", "yourselves", "could", "he'd", "he'll", "he's", "here's", "how's", "i'd", "i'll", "i'm", "i've", "let's", "ought", "she'd", "she'll", "that's", "there's", "they'd", "they'll", "they're", "they've", "we'd", "we'll", "we're", "we've", "what's", "when's", "where's", "who's", "why's", "would", "able", "abst", "accordance", "according", "accordingly", "across", "act", "actually", "added", "adj", "affected", "affecting", "affects", "afterwards", "ah", "almost", "alone", "along", "already", "also", "although", "always", "among", "amongst", "announce", "another", "anybody", "anyhow", "anymore", "anyone", "anything", "anyway", "anyways", "anywhere", "apparently", "approximately", "arent", "arise", "around", "aside", "ask", "asking", "auth", "available", "away", "awfully", "b", "back", "became", "become", "becomes", "becoming", "beforehand", "begin", "beginning", "beginnings", "begins", "behind", "believe", "beside", "besides", "beyond", "biol", "brief", "briefly", "c", "ca", "came", "cannot", "can't", "cause", "causes", "certain", "certainly", "co", "com", "come", "comes", "contain", "containing", "contains", "couldnt", "date", "different", "done", "downwards", "due", "e", "ed", "edu", "effect", "eg", "eight", "eighty", "either", "else", "elsewhere", "end", "ending", "enough", "especially", "et", "etc", "even", "ever", "every", "everybody", "everyone", "everything", "everywhere", "ex", "except", "f", "far", "ff", "fifth", "first", "five", "fix", "followed", "following", "follows", "former", "formerly", "forth", "found", "four", "furthermore", "g", "gave", "get", "gets", "getting", "give", "given", "gives", "giving", "go", "goes", "gone", "got", "gotten", "h", "happens", "hardly", "hed", "hence", "hereafter", "hereby", "herein", "heres", "hereupon", "hes", "hi", "hid", "hither", "home", "howbeit", "however", "hundred", "id", "ie", "im", "immediate", "immediately", "importance", "important", "inc", "indeed", "index", "information", "instead", "invention", "inward", "itd", "it'll", "j", "k", "keep", "keeps", "kept", "kg", "km", "know", "known", "knows", "l", "largely", "last", "lately", "later", "latter", "latterly", "least", "less", "lest", "let", "lets", "like", "liked", "likely", "line", "little", "'ll", "look", "looking", "looks", "ltd", "made", "mainly", "make", "makes", "many", "may", "maybe", "mean", "means", "meantime", "meanwhile", "merely", "mg", "might", "million", "miss", "ml", "moreover", "mostly", "mr", "mrs", "much", "mug", "must", "n", "na", "name", "namely", "nay", "nd", "near", "nearly", "necessarily", "necessary", "need", "needs", "neither", "never", "nevertheless", "new", "next", "nine", "ninety", "nobody", "non", "none", "nonetheless", "noone", "normally", "nos", "noted", "nothing", "nowhere", "obtain", "obtained", "obviously", "often", "oh", "ok", "okay", "old", "omitted", "one", "ones", "onto", "ord", "others", "otherwise", "outside", "overall", "owing", "p", "page", "pages", "part", "particular", "particularly", "past", "per", "perhaps", "placed", "please", "plus", "poorly", "possible", "possibly", "potentially", "pp", "predominantly", "present", "previously", "primarily", "probably", "promptly", "proud", "provides", "put", "q", "que", "quickly", "quite", "qv", "r", "ran", "rather", "rd", "recreation", "readily", "really", "recent", "recently", "ref", "refs", "regarding", "regardless", "regards", "related", "relatively", "research", "respectively", "resulted", "resulting", "results", "right", "run", "said", "saw", "say", "saying", "says", "sec", "section", "see", "seeing", "seem", "seemed", "seeming", "seems", "seen", "self", "selves", "sent", "seven", "several", "shall", "shed", "shes", "show", "showed", "shown", "showns", "shows", "significant", "significantly", "similar", "similarly", "since", "six", "slightly", "somebody", "somehow", "someone", "somethan", "something", "sometime", "sometimes", "somewhat", "somewhere", "soon", "sorry", "specifically", "specified", "specify", "specifying", "still", "stop", "strongly", "sub", "substantially", "successfully", "sufficiently", "suggest", "sup", "sure", "take", "taken", "taking", "tell", "tends", "th", "thank", "thanks", "thanx", "thats", "that've", "thence", "thereafter", "thereby", "thered", "therefore", "therein", "there'll", "thereof", "therere", "theres", "thereto", "thereupon", "there've", "theyd", "theyre", "think", "thou", "though", "thoughh", "thousand", "throug", "throughout", "thru", "thus", "til", "tip", "together", "took", "toward", "towards", "tried", "tries", "truly", "try", "trying", "ts", "twice", "two", "u", "un", "unfortunately", "unless", "unlike", "unlikely", "unto", "upon", "ups", "us", "use", "used", "useful", "usefully", "usefulness", "uses", "using", "usually", "v", "value", "various", "'ve", "via", "viz", "vol", "vols", "vs", "w", "want", "wants", "wasnt", "way", "wed", "welcome", "went", "werent", "whatever", "what'll", "whats", "whence", "whenever", "whereafter", "whereas", "whereby", "wherein", "wheres", "whereupon", "wherever", "whether", "whim", "whither", "whod", "whoever", "whole", "who'll", "whomever", "whos", "whose", "widely", "willing", "wish", "within", "without", "wont", "words", "world", "wouldnt", "www", "x", "yes", "yet", "youd", "youre", "z", "zero", "a's", "ain't", "allow", "allows", "apart", "appear", "appreciate", "appropriate", "associated", "best", "better", "c'mon", "c's", "cant", "changes", "clearly", "concerning", "consequently", "consider", "considering", "corresponding", "course", "currently", "definitely", "described", "despite", "entirely", "exactly", "example", "going", "greetings", "hello", "help", "hopefully", "ignored", "inasmuch", "indicate", "indicated", "indicates", "inner", "insofar", "it'd", "keep", "keeps", "novel", "presumably", "reasonably", "second", "secondly", "sensible", "serious", "seriously", "sure", "t's", "third", "thorough", "thoroughly", "three", "well", "wonder", "a", "about", "above", "above", "across", "after", "afterwards", "again", "against", "all", "almost", "alone", "along", "already", "also", "although", "always", "am", "among", "amongst", "amoungst", "amount", "an", "and", "another", "any", "anyhow", "anyone", "anything", "anyway", "anywhere", "are", "around", "as", "at", "back", "be", "became", "because", "become", "becomes", "becoming", "been", "before", "beforehand", "behind", "being", "below", "beside", "besides", "between", "beyond", "bill", "both", "bottom", "but", "by", "call", "can", "cannot", "cant", "co", "con", "could", "couldnt", "cry", "de", "describe", "detail", "do", "done", "down", "due", "during", "each", "eg", "eight", "either", "eleven", "else", "elsewhere", "empty", "enough", "etc", "even", "ever", "every", "everyone", "everything", "everywhere", "except", "few", "fifteen", "fify", "fill", "find", "fire", "first", "five", "for", "former", "formerly", "forty", "found", "four", "from", "front", "full", "further", "get", "give", "go", "had", "has", "hasnt", "have", "he", "hence", "her", "here", "hereafter", "hereby", "herein", "hereupon", "hers", "herself", "him", "himself", "his", "how", "however", "hundred", "ie", "if", "in", "inc", "indeed", "interest", "into", "is", "it", "its", "itself", "keep", "last", "latter", "latterly", "least", "less", "ltd", "made", "many", "may", "me", "meanwhile", "might", "mill", "mine", "more", "moreover", "most", "mostly", "move", "much", "must", "my", "myself", "name", "namely", "neither", "never", "nevertheless", "next", "nine", "no", "nobody", "none", "noone", "nor", "not", "nothing", "now", "nowhere", "of", "off", "often", "on", "once", "one", "only", "onto", "or", "other", "others", "otherwise", "our", "ours", "ourselves", "out", "over", "own", "part", "per", "perhaps", "please", "put", "rather", "re", "same", "see", "seem", "seemed", "seeming", "seems", "serious", "several", "she", "should", "show", "side", "since", "sincere", "six", "sixty", "so", "some", "somehow", "someone", "something", "sometime", "sometimes", "somewhere", "still", "such", "system", "take", "ten", "than", "that", "the", "their", "them", "themselves", "then", "thence", "there", "thereafter", "thereby", "therefore", "therein", "thereupon", "these", "they", "thickv", "thin", "third", "this", "those", "though", "three", "through", "throughout", "thru", "thus", "to", "together", "too", "top", "toward", "towards", "twelve", "twenty", "two", "un", "under", "until", "up", "upon", "us", "very", "via", "was", "we", "well", "were", "what", "whatever", "when", "whence", "whenever", "where", "whereafter", "whereas", "whereby", "wherein", "whereupon", "wherever", "whether", "which", "while", "whither", "who", "whoever", "whole", "whom", "whose", "why", "will", "with", "within", "without", "would", "yet", "you", "your", "yours", "yourself", "yourselves", "the", "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z", "A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z", "co", "op", "research-articl", "pagecount", "cit", "ibid", "les", "le", "au", "que", "est", "pas", "vol", "el", "los", "pp", "u201d", "well-b", "http", "volumtype", "par", "0o", "0s", "3a", "3b", "3d", "6b", "6o", "a1", "a2", "a3", "a4", "ab", "ac", "ad", "ae", "af", "ag", "aj", "al", "an", "ao", "ap", "ar", "av", "aw", "ax", "ay", "az", "b1", "b2", "b3", "ba", "bc", "bd", "be", "bi", "bj", "bk", "bl", "bn", "bp", "br", "bs", "bt", "bu", "bx", "c1", "c2", "c3", "cc", "cd", "ce", "cf", "cg", "ch", "ci", "cj", "cl", "cm", "cn", "cp", "cq", "cr", "cs", "ct", "cu", "cv", "cx", "cy", "cz", "d2", "da", "dc", "dd", "de", "df", "di", "dj", "dk", "dl", "do", "dp", "dr", "ds", "dt", "du", "dx", "dy", "e2", "e3", "ea", "ec", "ed", "ee", "ef", "ei", "ej", "el", "em", "en", "eo", "ep", "eq", "er", "es", "et", "eu", "ev", "ex", "ey", "f2", "fa", "fc", "ff", "fi", "fj", "fl", "fn", "fo", "fr", "fs", "ft", "fu", "fy", "ga", "ge", "gi", "gj", "gl", "go", "gr", "gs", "gy", "h2", "h3", "hh", "hi", "hj", "ho", "hr", "hs", "hu", "hy", "i", "i2", "i3", "i4", "i6", "i7", "i8", "ia", "ib", "ic", "ie", "ig", "ih", "ii", "ij", "il", "in", "io", "ip", "iq", "ir", "iv", "ix", "iy", "iz", "jj", "jr", "js", "jt", "ju", "ke", "kg", "kj", "km", "ko", "l2", "la", "lb", "lc", "lf", "lj", "ln", "lo", "lr", "ls", "lt", "m2", "ml", "mn", "mo", "ms", "mt", "mu", "n2", "nc", "nd", "ne", "ng", "ni", "nj", "nl", "nn", "nr", "ns", "nt", "ny", "oa", "ob", "oc", "od", "of", "og", "oi", "oj", "ol", "om", "on", "oo", "oq", "or", "os", "ot", "ou", "ow", "ox", "oz", "p1", "p2", "p3", "pc", "pd", "pe", "pf", "ph", "pi", "pj", "pk", "pl", "pm", "pn", "po", "pq", "pr", "ps", "pt", "pu", "py", "qj", "qu", "r2", "ra", "rc", "rd", "rf", "rh", "ri", "rj", "rl", "rm", "rn", "ro", "rq", "rr", "rs", "rt", "ru", "rv", "ry", "s2", "sa", "sc", "sd", "se", "sf", "si", "sj", "sl", "sm", "sn", "sp", "sq", "sr", "ss", "st", "sy", "sz", "t1", "t2", "t3", "tb", "tc", "td", "te", "tf", "th", "ti", "tj", "tl", "tm", "tn", "tp", "tq", "tr", "ts", "tt", "tv", "tx", "ue", "ui", "uj", "uk", "um", "un", "uo", "ur", "ut", "va", "wa", "vd", "wi", "vj", "vo", "wo", "vq", "vt", "vu", "x1", "x2", "x3", "xf", "xi", "xj", "xk", "xl", "xn", "xo", "xs", "xt", "xv", "xx", "y2", "yj", "yl", "yr", "ys", "yt", "zi", "zz"])

def cleanhtml(raw_html) -> str:
    cleanr = re.compile("<.*?>")
    cleantext = re.sub(cleanr, "", raw_html)
    return cleantext


class InitCommand(Command):

    _DOWNLOAD_URL = "https://ridb.recreation.gov/downloads/RIDBFullExport_V1_CSV.zip"

    name = "init"
    description = "Update the campground id search index"

    help = """The <question>init</> command is for rebuilding the campground database, which stores the campground names, ids, and their corresponding descriptions. Running this command is only necessary if there is a problem with the existing database or it needs to be updated.  

<options=bold>Examples:</>

Download and (re)build the campground databse:

    $ <info>poetry run rgov init</>
"""

    def ensure_download_dir_exists(self):
        os.makedirs(locations.DOWNLOAD_FOLDER, exist_ok=True)

    def request_ridb_data(self):
        urllib.request.urlretrieve(self._DOWNLOAD_URL, locations.DOWNLOAD_PATH)

    def unzip_ridb_data(self):
        with zipfile.ZipFile(locations.DOWNLOAD_PATH, "r") as zip_ref:
            zip_ref.extractall(locations.DOWNLOAD_FOLDER)

    def generate_sqlite_db(self):
        os.makedirs(locations.DATA_FOLDER, exist_ok=True)
        facilities_list = []
        with open(locations.FACILITIES_CSV_PATH, "r") as in_file:
            reader = csv.reader(in_file)
            for row in reader:
                # filter for reservable campgrounds with a non-empty name entry
                if row[7] == "Campground" and row[19] == "true" and row[5]:
                    words = cleanhtml(row[6]).lower().split()
                    descriptions = ' '.join([word for word in words if word not in STOPWORDS])
                    facilities_list.append(
                        [row[0], row[5].lower(), descriptions] 
                        )
                    
        # sort the entries alphabetically by name so search results are
        # also alphabetical
        facilities_list.sort(key=lambda x: x[1])

        if os.path.exists(os.path.join(locations.DATA_FOLDER, "rgov.db")):
            os.remove(os.path.join(locations.DATA_FOLDER, "rgov.db"))

        with contextlib.closing(sqlite3.connect(locations.FACILITIES_DB)) as con, con, \
            contextlib.closing(con.cursor()) as cur:
            cur.execute('''CREATE TABLE campgrounds
                           (id, name, description)''')
            sql_statement = 'INSERT INTO campgrounds VALUES (?, ?, ?)'
            cur.executemany(sql_statement, facilities_list)
            con.commit()

    def delete_temp_files(self):
        """
        Removes the project-local downlaod folder
        and its contents.
        """
        shutil.rmtree(locations.DOWNLOAD_FOLDER)

    def handle(self) -> int:
        steps = [
            ("Checking download directory", self.ensure_download_dir_exists),
            (f"Downloading data from {self._DOWNLOAD_URL}", self.request_ridb_data),
            ("Unzipping", self.unzip_ridb_data),
            ("Generating database", self.generate_sqlite_db),
            ("Deleting temporary files", self.delete_temp_files),
        ]

        if not self.confirm(
            "Continue with downloading and generating the campsite id index?", False
        ):
            return

        for step in steps:
            format = f"<question>{step[0]}...</question>"
            self.line("")
            self.write(format)
            step[1]()
            self.overwrite(format + f"<comment>done</comment>.")

        self.line("")
        return 0



import datetime
import getpass
import logging
import os
import time

import daemon

from cleo import Command
from cleo.helpers import option, argument

from rgov import locations
from rgov import pushsafer
from rgov.campground import Campground
from rgov.dates import Dates


class DaemonCommand(Command):

    name = "daemon"
    description = "Start a daemon that checks for availablity automatically"

    help = """The <question>daemon</> command starts a Unix daemon that checks for campground availability every five minutes. If one or more campground(s) are found to have available sites, a Pushsafer notification is sent with a summary of which campground(s) are currently available.

Note that a Pushsafer account and API key is required to use this command, and devices (e.g. a phone) must be configured for for it to work.

<options=bold>Examples:</>

Check if North Rim Campground has available sites on February 2nd, 2022 for 3 nights, and quit after four notifications have been sent:

    $ <info>rgov daemon --notify-limit 4 2-2-2022 3 232489</>

Check if North Rim and Spring Canyon campgrounds have available sites on March 20th, 2022 for five nights:

    $ <info>rgov daemon 3-20-2022 5 232489 234064</>
"""

    arguments = [
        argument("date", "The date of your arrival (mm-dd-yyyy)"),
        argument("length", "The length of stay in nights"),
        argument("id", "The campground id(s) to check", multiple=True),
    ]

    options = [
        option(
            "notify-limit",
            "N",
            "The number of notifications to send before the program exits [3]",
            flag=False,
            value_required=True,
        ),
        option(
            "command",
            "c",
            "Command to run if availability found",
            flag=False,
            value_required=True,
        ),
    ]

    def handle(self) -> int:
        id_input = self.argument("id")
        date_input = self.argument("date")
        length_input = self.argument("length")
            
        if self.option("notify-limit"):
            notify_limit = int(self.option("notify-limit"))
        else:
            notify_limit = 3
            
        dates = Dates(date_input, length_input)
        campgrounds = [Campground(id) for id in id_input]

        # make sure the api key works
        if os.path.exists(locations.AUTH_FILE):
            ps_username, ps_api_key = pushsafer.read_credentials()
            authenticated = pushsafer.validate_key(ps_username, ps_api_key)
        else:
            authenticated = False
        while authenticated == False:
            ps_username, ps_api_key = pushsafer.input_credentials()
            authenticated = pushsafer.validate_key(ps_username, ps_api_key)
            if authenticated == False:
                self.line("Invalid credentials.")
            else:
                if self.confirm("Save for future use?", False):
                    pushsafer.write_credentials(ps_username, ps_api_key)

        self.line("<fg=magenta>Starting to check.</fg=magenta>")
        with daemon.DaemonContext():
            logging.basicConfig(filename=locations.LOG_FILE,
                                filemode='a',
                                format='[%(asctime)s] %(message)s',
                                datefmt='%Y/%d/%m %H:%M:%S',
                                level=logging.INFO)
            logging.info('starting to search')
            
            notification_counter = 0
            
            while True:
                logging.info('------------checking------------')
                available = {} 
                for campground in campgrounds:
                    try:
                        campground.get_available(dates.request_dates, dates.stay_dates)

                    except (HTTPError, IndexError) as error:
                        logging.error(error)
                        time.sleep(10)
                        continue
                        
                    if len(campground.available) > 0:
                        logging.info(f'{campground.name} - found available site(s)')
                        available[campground.name] = campground.available
                    else:
                        logging.info(f'{campground.name} - no available site(s)')

                    if campground != campgrounds[-1]:
                        time.sleep(1)

                if available:
                    message = pushsafer.gen_notifier_text(available)
                    pushsafer_status = pushsafer.notify(ps_api_key, 'a', message)
                    if pushsafer_status['status'] == 0:
                        logging.error(f"Pushsafer: {pushsafer_status}")
                    else:
                        logging.info(f"Pushsafer: {pushsafer_status}")
                    notification_counter += 1
                if notification_counter == notify_limit:
                    reached_notify_limit = ("notification limit reached "
                                            f"[{notification_counter}/" 
                                            f"{notify_limit}] - exiting")
                    logging.info(reached_notify_limit)
                    return 0
                                           
                time.sleep(300)  # wait 5 minutes before rechecking



import datetime

import time

from urllib.error import HTTPError

from cleo import Command
from cleo.helpers import argument, option

from rgov import pushsafer
from rgov.dates import Dates
from rgov.campground import Campground

class CheckCommand(Command):
    name = "check"
    description = "Check campground(s) for availability"
    arguments = [
        argument("date",
                 "The date if arrival (mm-dd-yyyy)"),
        argument("length",
                 "Length of stay in nights"),
        argument("id",
                 "The campground id(s) to check",
                 multiple=True),
    ]
    options = [
        option("cron-mode", "c", "Run once and notify if availability found"),
        option("url", "u", "Print the campground url(s) with the output"),
    ]


    help = """The <question>check</> command prints out a summary of campsite availability for the the given campground(s) over the specified date range. 

All three arguments are required, and the campground id(s) must be the final argument. The id(s) can be found using the <question>search</> command or by finding the number that is in the url on the corresponding campground's www.recreation.gov webpage.  

The cron-mode option will send a Pushsafer notification (if set up). It is intended as a replacement for the <question>daemon</> command, if running a daemon is not desirable or the system is not Unix.  

<options=bold>Examples:</>

Check if North Rim Campground has available sites on February 2nd, 2022 for 3 nights:

    $ <info>rgov check 2-2-2022 3 232489</>

Check if North Rim and Spring Canyon campgrounds have available sites on March 20th, 2022 for 4 nights:

    $ <info>rgov check 3-20-2022 4 232489 234064</>
"""
    
    def handle(self) -> int:
        ids_input = self.argument("id")
        date_input = self.argument("date")
        length_input = self.argument("length")

        campgrounds = [Campground(id) for id in ids_input]
        dates = Dates(date_input, length_input)

        column_width = max([len(c.name) for c in campgrounds])

        for campground in campgrounds:
            try:
                campground.get_available(dates.request_dates, dates.stay_dates)

            except (HTTPError, IndexError) as error:
                self.line(campground.gen_cli_text(column_width, error))
                continue

            self.line(campground.gen_cli_text(column_width))
                
        if self.option("cron-mode"):
            self.line("Not yet implemented.")                

        return 0       




import datetime
import time

from urllib.error import HTTPError

from cleo.helpers import option, argument
from cleo import Command

from rgov.campground import Campground
from rgov.dates import Dates
from rgov.search import search 

class RunCommand(Command):

    def add_campgrounds(self):
        campgrounds = {}
        search_input = self.ask("Search for campgrounds:")
        if not search_input:
            return {}
        search_input_list = search_input.split(" ")

        if search_input_list[-1] == "-d":
            descriptions = True 
            del search_input_list[-1]
        else:
            descriptions = False 

        search_results = search(search_input_list, descriptions)
        search_results = {name: num for name, num in search_results}

        if search_results:
            search_results["(none of the above)"] = None
            search_input = self.choice('Select campground(s)',
                                       list(search_results.keys()),
                                       multiple=True)
            for campground in search_input:
                if campground == "(none of the above)":
                    pass
                else:
                    id_num = search_results[campground]
                    campgrounds[campground] = Campground(id_num)
        else:
            no_results_msg = (f"No results for {search_input}. "
                               "Try appending \"-d\".")
            self.line(no_results_msg)

        return campgrounds 

    def selections_empty(self, campgrounds: dict):
        if not campgrounds:
            self.line("Nothing to do.")
            return True 
        else:
            return False 

    def print_selections(self, campgrounds):
        self.line("\n<fg=yellow>Currently selected</>:")
        for campground in campgrounds:
             self.line(f"<info>{campground}</>")

    name = "run"
    description = "Run interactively"

    help = """The <question>run</> command is an interactive alternative to the <question>search</>, <question>check</>, and <question>daemon</> commands. 

Searching for campgrounds and setting the dates are done via prompt. If any or all of the selected campgrounds are unavailable, a confirmation prompt will ask if the <question>daemon</> should be started, which will check for availability every five minutes and send a notification via Pushsafer if availability is found (a Pushsafer account an API key is required for this).  

When searching for campgrounds interactively, note that a "-d" appended to the search terms will check for matching campground descriptions, whereas the default is to search for campgrounds by name (more information on description vs. name searches can be found in the help section for the <question>search</> command). 
"""
    def handle(self):
        campgrounds = self.add_campgrounds()

        if not campgrounds:
            campgrounds = self.add_campgrounds()

        if self.selections_empty(campgrounds):
            return 0

        self.print_selections(campgrounds)

        choice = self.ask("\nWhat now? ([a]dd [r]emove [c]ontinue):")
        while choice != "c":
            if choice == "a":
                new = self.add_campgrounds()
                campgrounds.update(new) 

            elif choice == "r":
               subtract = self.choice("Select a campground(s) to remove:",
                                    list(campgrounds.keys()),
                                    multiple=True)
               for item in subtract:
                   del campgrounds[item]

            self.print_selections(campgrounds)
            choice = self.ask("\nWhat now? (a)dd (r)emove (c)ontinue?")

        if self.selections_empty(campgrounds):
            return 0

        self.line("")
        column_width = max(map(len, campgrounds))

        def month_validator(num):
            if int(num) not in range(1,13):
                raise Exception("Not a digit from 1-12.")
            return num

        def day_validator(num):
            if int(num) not in range(1,31):
                raise Exception("Not a digit from 1-31.")
            return num

        def year_validator(num):
            this_year = datetime.datetime.today().year
            if int(num) not in range(this_year, this_year + 1):
                raise Exception("Not a valid four digit year.")
            return num

        month = self.create_question('Enter month of arrival:')
        month.set_validator(month_validator)
        month = self.ask(month)

        day = self.create_question('Enter day of arrival:')
        day.set_validator(day_validator)
        day = self.ask(day)

        year = self.create_question('Enter year of arrival:')
        year.set_validator(year_validator)
        year = self.ask(year)

        length_input = self.ask('Enter number of nights:')
        self.line("")

        date_input = f"{month}-{day}-{year}"

        dates = Dates(date_input, length_input)
        unavailable = []

        self.line("<fg=yellow>Availability</>:")

        for campground in campgrounds.values():
            try:
                campground.get_available(dates.request_dates, dates.stay_dates)

            except (HTTPError, IndexError) as error:
                unavailable.append(campground.id_num)
                self.line(campground.gen_cli_text(column_width, error)) 
                time.sleep(2)
                continue

            self.line(campground.gen_cli_text(column_width))
                      
            if len(campground.available) == 0:
                unavailable.append(campground.id_num)

        if unavailable:
            self.line("")
            daemon_question = ("One or more campground(s) unavailable. "
                               "Start rgov daemon?")

            if not self.confirm(daemon_question, False):
                return
            else:
                ids = " ".join(unavailable)
                args = f"{date_input} {length_input} {ids}"
                self.call('daemon', args)



import sys

from cleo import Application

from rgov.commands import check, search, initialize, check_daemon, run

commands = [
    check.CheckCommand(),
    search.SearchCommand(),
    initialize.InitCommand(),
    run.RunCommand(),
    check_daemon.DaemonCommand(),
]

application = Application()

[application.add(command) for command in commands]

def main():
    application.run()

if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['rgov', 'rgov.commands']

package_data = \
{'': ['*']}

install_requires = \
['cleo>=0.8.1,<0.9.0',
 'fake-useragent>=0.1.11,<0.2.0',
 'python-daemon>=2.3.0,<3.0.0']

entry_points = \
{'console_scripts': ['rgov = rgov.application:main']}

setup_kwargs = {
    'name': 'rgov',
    'version': '0.6.5',
    'description': 'Check if campgrounds have availability on Recreation.gov',
    'long_description': '\n\n# rgov - Recreation.gov Campground Checker\n\n[![img](https://img.shields.io/badge/License-GPL%20v3-blue.svg)](http://www.gnu.org/licenses/gpl-3.0)\n\n`rgov` is a command line program to check for campground availability on Recreation.gov. It is intended to be easy to use, and provides an interactive mode for easily searching for and checking multiple campgrounds for availability at once. It can also be used as a unix daemon that sends push notifications to your phone or other device when available sites are found.  \n\n## Installation\n\nRequires: Python 3.6+\n\nFrom pypi:\n\n$ `pip install rgov`\n\nManually:\n\n$ `git clone https://github.com/jsbmg/rgov`\n\n$ `cd rgov`\n\n$ `pip install . pyproject.toml`\n\nIn order to receive notifications, you will need to make an account with Pushsafer and configure it for use with your phone or device of choice. \n\nhttps://www.pushsafer.com/\n\n## Quick Start\nThe easiest way to run rgov is with the `run` command. Use it to build a list of campgrounds, enter your dates of stay, and start checking for available sites:\n\n`$ rgov run` \n\nThe `search`, `check`, and `daemon` commands provide the same functionality as above but non-interactively.\n\nRun $ `rgov` to see a list of all commands, and $ `rgov help <command>` to see more information about each one. \n',
    'author': 'Jordan Sweet',
    'author_email': 'hello@jordandsweet.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



import contextlib
import json
import sqlite3

from collections import defaultdict
from urllib.error import HTTPError
from urllib.parse import urlencode
from urllib.request import Request, urlopen

from fake_useragent import UserAgent

from rgov import locations


class AvailabilityNotFoundError(Exception):
    def __init__(self, arg=None):
        if arg:
            self.message = arg
        else:
            self.message = ("Try initializing this attribute"  
                            "with the get_availability() method first.")
    def __str__(self):
        return self.message


class Campground:

    def __init__(self, id_num): 
        self.id_num = id_num
        self._name = None 
        self._available = None 
        self._url = None
        self._cli_text = None

    def _request(self, request_dates: list, stay_dates: list) -> list:
        endpoint = "https://www.recreation.gov/api/camps/availability/campground"
        requests = []
        for date in request_dates:
            url = f"{endpoint}/{self.id_num}/month?"
            params = {"start_date": date}
            query_string = urlencode(params)
            url = url + query_string
            request = Request(url)
            request.add_header("User-Agent", UserAgent().random)
            try:
                data = urlopen(request)
            except HTTPError:
                raise
            data = data.read()
            data_loaded = json.loads(data)
            try:
                campsite_data = data_loaded["campsites"]
            except KeyError:
                raise
            requests.append(campsite_data)
        return requests

    def get_available(self, request_dates: list, stay_dates: list):
        try:
            requests = self._request(request_dates, stay_dates)   
        except (HTTPError, KeyError):
            raise

        d = defaultdict(int) 
        for request in requests:
            for site in request.values():
                for date in stay_dates:
                    if date in site["availabilities"]:
                        if site["availabilities"][date] == "Available":
                            d[site["site"]] += 1

        self._available = [k for k, v in d.items() if v == len(stay_dates)] 

    @property
    def name(self):
        if not self._name == None:
            return self._name
        else:
            sql_statement = """SELECT name FROM campgrounds WHERE id = (?)"""
            try:
                with contextlib.closing(sqlite3.connect(locations.FACILITIES_DB)) as con, con, \
                        contextlib.closing(con.cursor()) as cur:
                        cur.execute(sql_statement, (self.id_num,))
                        self._name = cur.fetchone()[0].title()
                        if not self._name:
                            raise ValueError(f"{self.id_num} is not a valid id.")
                        return self._name
            except sqlite.OperationalError:
                raise sqlite3.OperationalError(("Something went wrong "
                                                "with the database: "
                                                "Try running `rgov init`."))

    @property
    def available(self):
        if not self._available == None:
            return self._available
        else:
            raise AvailabilityNotFoundError

    @property
    def url(self):
        if not self._url == None:
            return self._url
        else:
            base_url = "https://www.recreation.gov/camping/campgrounds"
            self._url = f"{base_url}/{self.id_num}/availability"
            return self._url

    def gen_cli_text(self, width=None, error=None):
        col_1 = f"<info>{self.name}</>:"
        len_name = len(self.name)

        if not width:
            width = len(self.name)

        width += len(col_1) - len_name  

        if error:
            col_2 = f"<error>{error}</>"
        else:
            try:
                num_available_sites = len(self.available)
            except AvailabilityNotFoundError:
                raise

            if 1 <= num_available_sites <= 12:
                sorted_sites = ", ".join(self.available)
                col_2 = f"<fg=cyan>{sorted_sites}</> available"

            elif num_available_sites > 12:
                col_2 = f"<fg=magenta>{num_available_sites}</> sites available"

            else:
                col_2 = f"<fg=red>full</>"

        self._cli_text = f"{col_1:{width}} {col_2}" 
        return self._cli_text 



from cleo import Command
from cleo.helpers import option, argument

from rgov import search


class SearchCommand(Command):

    name = "search"
    description = "Search for campground ids"
    arguments = [argument("terms", "The terms to search for", multiple=True)]
    options = [
        option("descriptions", "d", "Search descriptions"),
        option("interactive", "i", "Interact with the search results")
    ]

    help = """The <question>search</> command searches a local database for campground id(s). By default, search arguments are matched with the name of the campground, but the <comment>descriptions</> option can be enabled to search for matching descriptions.

The campground index can always be rebuilt or updated via the <question>init</> command.

<options=bold>Examples:</>

Search for all campgrounds associated with the Grand Canyon:

    $ <info>rgov search -d grand canyon</>

Search for the campground "North Rim Campground":

    $ <info>rgov search north rim</>
"""
    
    def handle(self):
        search_terms = self.argument("terms")
        
        if self.option("descriptions"):
            descriptions = True
        else:
            descriptions = False 
            
        search_results = search.search(search_terms, descriptions)

        for name, id_num in search_results:
            self.line(f"<info>{name}</> [<question>{id_num}</>]") 



import getpass
import json
import os
from pathlib import Path

from urllib.parse import urlencode
from urllib.request import Request, urlopen

from rgov import locations

def input_credentials() -> tuple[str, str]:
    username = input("Enter username/email for Pushsafer: ")
    key = getpass.getpass("Enter API key for Pushsafer: ")
    return username, key

def write_credentials(email, key, passcmd=False):
    os.makedirs(os.path.join(locations.CONFIG_DIR, "rgov"), exist_ok=True)
    os.umask(0)
    if passcmd:
        print("Not yet implemented.")
    with open(os.open(locations.AUTH_FILE, os.O_CREAT | os.O_WRONLY, 0o600), 'w') as file: 
        file.truncate(0)
        file.write(f"{email}\n")
        # if passcmd:
        #    file.write(f"passcmd: {key}")
        # else:
        file.write(f"{key}")

def read_credentials():
    with open(locations.AUTH_FILE, 'r') as file:
        lines = file.readlines()
        lines = [line.strip() for line in lines]
        username = lines[0]
        key = lines[1]
        return username, key

def validate_key(username: str, key: str) -> bool:
    url = 'https://www.pushsafer.com/api-k'
    post_fields = {
        "u" : username,
        "k" : key,
        }
    request = Request(url, urlencode(post_fields).encode())
    status = json.loads(urlopen(request).read().decode())
    if status['status'] == 1:
        return True
    else:
        return False

def gen_notifier_text(campground_results: dict) -> str:
    message = ""
    for name, available in campground_results.items():
        num_sites = len(available)
        available_str = ", ".join(available)
        if 0 < num_sites < 10:
            message += f"{name}: site(s) {available_str} available!\n"
        elif num_sites > 10:
            message += f"{name}: {num_sites} sites available!\n"
    return message


def notify(key: str, device:str, message: str, url=None) -> dict:
    endpoint = 'https://www.pushsafer.com/api'
    post_fields = {
        "d" : device,
        "t" : 'Campsite Availability Update',
        "m" : message,
        "s" : 11,
        "v" : 3,
        "i" : 33,
        "c" : '#FF0000',
        "k" : key,
    }
    if url:
        post_fields['u'] = url
        post_fields['ut'] = 'Campsite Page'

    request = Request(endpoint, urlencode(post_fields).encode())
    status = json.loads(urlopen(request).read().decode())
    return status
